@@ -4,10 +4,12 @@ import pickle
 import pandas as pd
 import pygit2 as git2
 import git
+import sys
 import git2net
 import shutil
 from pathlib import Path
 import stat
+import subprocess
 import numpy
 from .utility import Utility
 
@@ -42,14 +44,14 @@ class Version():
     -------
     handleError(func, path, exc_info)
         Error handler function which will try to change file permission and call the calling function again.
-    clone_repository(repo, data_root_dir, github_token=None)
+    clone_repository(repo, data_root_dir, github_token=None, new_clone=False):
         Cloning repository from git.
     generate_data_base(data_root_dir)
         Extracting version data from a local repository and storing them in a mysql data base.
-    generate_version_pandas_tables(repo, data_root_dir)
+    generate_version_pandas_tables(repo, data_root_dir, check_for_updates=True)
         Extracting edits and commits in a pandas table.
-    define_unknown_users(user_list, data_root_dir)
-        Define unknown users in commits pandas table.
+    define_unknown_user(unknown_user_name, uuid, data_root_dir, new_user=False)
+        Define unknown user in commits pandas table.
     get_unknown_users(data_root_dir)
         Get all unknown users in from commits.
     get_version(data_root_dir, filename=VERSION_COMMITS)
@@ -99,7 +101,7 @@ class Version():
             func(path)
 
     @staticmethod
-    def clone_repository(repo, data_root_dir, github_token=None):
+    def clone_repository(repo, data_root_dir, github_token=None, new_clone=False):
         """
         Clone_repository(repo, data_root_dir, github_token=None)
 
@@ -113,6 +115,8 @@ class Version():
             Repo dir of the project.
         github_token : str
             Token string.
+        new_clone : bool, default=True
+            Initiating a completely new clone of the repository
 
         Notes
         -----
@@ -127,6 +131,17 @@ class Version():
         version_folder.mkdir(parents=True, exist_ok=True)
 
         repo_dir = version_folder.joinpath(Version.VERSION_REPOSITORY_DIR)
+        if (repo_dir.exists ()) & (not new_clone):
+            old_path = Path.cwd()
+            os.chdir(repo_dir)
+            try:
+                git2output = subprocess.check_output(["git", "pull"])
+            except:
+                print("This repository is empty, git pull generates an error")
+                print('git said:', git2output)
+            os.chdir(old_path)
+            return
+
         if repo_dir.exists ():
             shutil.rmtree(repo_dir.resolve(), onerror=Version.handleError)
 
@@ -155,7 +170,7 @@ class Version():
                     print(" -> An exception occurred")
 
     @staticmethod
-    def generate_data_base(data_root_dir):
+    def generate_data_base(data_root_dir, new_extraction=False):
         """
         generate_data_base(data_root_dir)
 
@@ -165,6 +180,8 @@ class Version():
         ----------
         data_root_dir : str
             Data root directory for the repository.
+        new_extraction: bool, default = False
+            Start a new complete extraction run
         
         Notes
         -----
@@ -186,18 +203,18 @@ class Version():
         repo_dir = version_folder.joinpath(Version.VERSION_REPOSITORY_DIR)
         sqlite_db_file = version_folder.joinpath(Version.VERSION_DB)
 
-        if os.path.exists(sqlite_db_file):
+        if new_extraction & os.path.exists(sqlite_db_file):
             os.remove(sqlite_db_file)
 
         git2net.mine_git_repo(repo_dir, sqlite_db_file,
-                              extract_complexity=True,
-                              extract_text=True,
-                              no_of_processes=Version.no_of_proceses,
-                              all_branches=True,
-                              max_modifications=1000)
+                                extract_complexity=True,
+                                extract_text=True,
+                                no_of_processes=Version.no_of_proceses,
+                                all_branches=True,
+                                max_modifications=1000)
 
     @staticmethod
-    def generate_version_pandas_tables(repo, data_root_dir):
+    def generate_version_pandas_tables(repo, data_root_dir, check_for_updates=True):
         """
         generate_version_pandas_tables(repo, data_root_dir)
 
@@ -209,14 +226,21 @@ class Version():
             Repository object from pygithub.
         data_root_dir: str
             Data root directory for the repository.
+        check_for_updates : bool, default=True
+            Check first if there are any new pull requests information.
 
         """
 
+        if check_for_updates:
+            commits = repo.get_commits() 
+            old_commits = Version.get_version(data_root_dir, filename=Version.VERSION_COMMITS)
+            if not Utility.check_for_updates_paginated(commits, old_commits):
+                return
+
         Version.generate_data_base(data_root_dir)
 
         version_folder = Path(data_root_dir, Version.VERSION_DIR)
         sqlite_db_file = version_folder.joinpath(Version.VERSION_DB)
-        print("1")
         db = sqlite3.connect(sqlite_db_file)
         pd_commits = pd.read_sql_query("SELECT * FROM commits", db)
         pd_edits = pd.read_sql_query("SELECT * FROM edits", db)
@@ -245,8 +269,22 @@ class Version():
                     pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'author'] = author_id   
                     pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'committer'] = committer_id 
                     if (author_id is None) and (committer_id is None):
-                        pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'unknown_user'] = row.committer_name
-
+                        users = Utility.get_users(data_root_dir)
+                        found = False
+                        if "alias" in users:
+                            for index2, row2 in users.iterrows():
+                                if not pd.isnull(row2["alias"]) and row2["alias"] is not None:
+                                    all_alias = row2["alias"].split(';')
+                                    for alias in all_alias:
+                                        if commiter_name == alias:
+                                            pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'author'] = row2["anonym_uuid"] 
+                                            pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'committer'] = row2["anonym_uuid"]
+                                            found = True
+                                            break
+                                if found:
+                                    break
+                        if not found:
+                            pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'unknown_user'] = row.committer_name
             else:
                 commit_sha = pd_commits[pd_commits.committer_name == commiter_name].iloc[0].commit_sha 
                 author_id = Utility.extract_author_data_from_commit(repo, commit_sha, 
@@ -256,7 +294,22 @@ class Version():
                 pd_commits.loc[pd_commits.committer_name == commiter_name, 'author'] = author_id   
                 pd_commits.loc[pd_commits.committer_name == commiter_name, 'committer'] = committer_id 
                 if (author_id is None) and (committer_id is None):
-                    pd_commits.loc[pd_commits.committer_name == commiter_name, 'unknown_user'] = commiter_name 
+                    users = Utility.get_users(data_root_dir)
+                    found = False
+                    if "alias" in users:
+                        for index, row in users.iterrows():
+                            if not pd.isnull(row["alias"]) and row["alias"] is not None:
+                                all_alias = row["alias"].split(';')
+                                for alias in all_alias:
+                                    if commiter_name == alias:
+                                        pd_commits.loc[pd_commits.committer_name == commiter_name, 'author'] = row["anonym_uuid"] 
+                                        pd_commits.loc[pd_commits.committer_name == commiter_name, 'committer'] = row["anonym_uuid"]
+                                        found = True
+                                        break
+                            if found:
+                                break
+                    if not found:
+                        pd_commits.loc[pd_commits.committer_name == commiter_name, 'unknown_user'] = commiter_name 
         pd_commits.drop(['committer_name'], axis=1, inplace=True)  
 
         users = Utility.get_users(data_root_dir)
@@ -303,35 +356,33 @@ class Version():
             pickle.dump(pd_Branches, f)          
 
     @staticmethod
-    def define_unknown_users(user_dict, data_root_dir):
+    def define_unknown_user(unknown_user_name, uuid, data_root_dir, new_user=False):
         """
-        define_unknown_users(user_dict, data_root_dir)
+        define_unknown_user(unknown_user_name, uuid, data_root_dir, new_user=False)
 
-        Define unknown users in commits pandas table.
+        Define unknown user in commits pandas table.
 
         Parameters
         ----------
-        user_dict: dict
-            Dictionary which contains users. 
+        unknown_user_name: str
+            Name of unknown user. 
+        uuid: str
+            Uuid can be the anonym uuid of another user or random uuid for a new user. 
         data_root_dir : str
             Data root directory for the repository.
+        new_user : bool, default=False
+            A complete new user with uuid will be generated.
 
-        Notes
-        -----
-        Example User: {"unknown_user": "real user node id"}
-        If the real user node id does not exist in the users table then a new user will be created
-        
         """
         pd_commits = Version.get_version(data_root_dir)
         if "unknown_user" in pd_commits:
-            unknown_user_commits = pd_commits.loc[pd_commits.unknown_user.notna()]
-            unknown_users = unknown_user_commits.unknown_user.unique()
-            for unknown_user in unknown_users:
-                uuid = Utility.define_unknown_user(user_dict,unknown_user,data_root_dir)
-                if uuid is not None:
-                    pd_commits.loc[pd_commits.unknown_user == unknown_user, 'author'] = uuid
-                    pd_commits.loc[pd_commits.unknown_user == unknown_user, 'committer'] = uuid
-                    pd_commits.loc[pd_commits.unknown_user == unknown_user, 'unknown_user'] = numpy.NaN
+            unknown_users = pd_commits.unknown_user.unique()
+            if unknown_user_name in unknown_users:
+                new_uuid = Utility.define_unknown_user(unknown_user_name, uuid,data_root_dir, new_user=new_user)
+                if new_uuid is not None:
+                    pd_commits.loc[pd_commits.unknown_user == unknown_user_name, 'author'] = new_uuid
+                    pd_commits.loc[pd_commits.unknown_user == unknown_user_name, 'committer'] = new_uuid
+                    pd_commits.loc[pd_commits.unknown_user == unknown_user_name, 'unknown_user'] = numpy.NaN
 
             version_folder = Path(data_root_dir, Version.VERSION_DIR)
             pd_commits_file = Path(version_folder, Version.VERSION_COMMITS)
